/**
 * (c) 2017 Shoreditch Ops Ltd. All rights reserved.
 *
 * This file is part of Artillery Pro (https://artillery.io)
 *
 * NOTICE: All information contained herein is, and remains
 * the property of Shoreditch Ops Ltd. The intellectual and
 * technical concepts contained herein are proprietary to
 * Shoreditch Ops Ltd and are protected by copyright law.
 *
 * Modification and/or redistribution of this material is
 * strictly forbidden unless prior written permission is
 * obtained from Shoreditch Ops Ltd.
 *
 */

'use strict';

const path = require('path');
const fs = require('fs');

const chalk = require('chalk');

const tryResolve = require('try-require').resolve;
let pro = null;
if (tryResolve('artillery-pro')) {
  pro = require('artillery-pro');
};

const debug = require('debug')('plugin:sslClientAuth');

module.exports = {
  Plugin: SSLClientAuthPlugin
};

function SSLClientAuthPlugin(script, events, opts) {
  this._activated = (pro !== null && pro.isActivatedSync());

  if (!this._activated) {
    console.log(
      chalk.yellow(
        'Artillery',
        'Pro',
        'not detected.',
        'Please install',
        'and activate Artillery',
        'Pro.\nThis',
        'plugin must',
        'be used with Artillery',
        'Pro.'
      )
    );
  }

  const basePath = path.dirname(opts.absoluteScriptPath);

  script.config.__sslAuthPlugin = {};

  if (script.config.tls && script.config.tls.client) {
    loadIfDefined(script.config, 'pfx', basePath);
    loadIfDefined(script.config, 'key', basePath);
    loadIfDefined(script.config, 'cert', basePath);
    loadIfDefined(script.config, 'ca', basePath);

    script.config.__sslAuthPlugin.passphrase = script.config.tls.client.passphrase;

    if (!script.config.processor) {
      script.config.processor = {};
    }

    script.config.processor.sslClientAuthPluginCreateVariables = function(
      userContext,
      events,
      done
    ) {
      userContext.vars.tlsPfx = script.config.__sslAuthPlugin.pfx;
      userContext.vars.tlsKey = script.config.__sslAuthPlugin.key;
      userContext.vars.tlsCert = script.config.__sslAuthPlugin.cert;
      userContext.vars.tlsCa = script.config.__sslAuthPlugin.ca;
      userContext.vars.tlsPassphrase = script.config.__sslAuthPlugin.passphrase;

      return done();
    };

    script.config.processor.sslClientAuthPluginSetOpts = function(
      req,
      userContext,
      events,
      done
    ) {
      if (req.sslAuth !== false) {
        req.key = userContext.vars.tlsKey;
        req.cert = userContext.vars.tlsCert;
        req.ca = userContext.vars.tlsCa;
        req.pfx = userContext.vars.tlsPfx;
        req.passphrase = userContext.vars.tlsPassphrase;
      }
      return done();
    };

    script.scenarios.forEach(function(scenario) {
      if (!scenario.beforeScenario) {
        scenario.beforeScenario = [];
      }

      if (!scenario.beforeRequest) {
        scenario.beforeRequest = [];
      }

      scenario.beforeScenario.push('sslClientAuthPluginCreateVariables');
      scenario.beforeRequest.push('sslClientAuthPluginSetOpts');
    });

    debug('Plugin initialized');
  }

  return this;
}

function loadIfDefined(config, propName, basePath) {
  if (config.tls.client[propName]) {
    let filename = path.resolve(basePath, config.tls.client[propName]);
    config.__sslAuthPlugin[propName] = fs.readFileSync(filename, 'utf8');
  }
}
